package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.ZoneOffset;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEmployees();
            findEmployeeBySalary();
            findEmployeeByJoinDate();
            //setting parameter
            findEmployeeByJoinDate2();
            //not between
            findEmployeeByJoinDate3();
        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEmployees() {
        Employee employee1 = Employee.create("Diana", 3000, LocalDate.of(1999, 11, 15));
        Employee employee2 = Employee.create("Rose", 4000, LocalDate.of(2011, 5, 1));
        Employee employee3 = Employee.create("Denise", 1500, LocalDate.of(2006, 1, 10));
        Employee employee4 = Employee.create("Mike", 2000, LocalDate.of(2015, 8, 20));
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee1);
        em.persist(employee2);
        em.persist(employee3);
        em.persist(employee4);
        em.getTransaction().commit();
        em.close();
    }

    private static void findEmployeeBySalary() {
        System.out.println("-- Employee with salary BETWEEN 2000 and 4000 --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery(
                "SELECT e FROM Employee e WHERE e.salary BETWEEN 2000L AND 4000L order by e.salary");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeByJoinDate() {
        System.out.println("-- Employee with join date BETWEEN '1990-01-01' and '2010-05-01' --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery(
                "SELECT e FROM Employee e WHERE e.joinDate BETWEEN '1990-01-01' AND '2010-05-01' order "
                        + "by e.joinDate");
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeByJoinDate2() {
        System.out.println("-- Employee with join date BETWEEN '2005-01-01' and '2018-01-01' --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery(
                "SELECT e FROM Employee e WHERE e.joinDate BETWEEN :startDate AND :endDate order "
                        + "by e.joinDate");
        query.setParameter("startDate", localToTimeStamp(LocalDate.of(2005,1,1)));
        query.setParameter("endDate", localToTimeStamp(LocalDate.of(2018,1,1)));
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeByJoinDate3() {
        System.out.println("-- Employee with join date NOT BETWEEN'2005-01-01' and '2018-01-01' --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery(
                "SELECT e FROM Employee e WHERE e.joinDate NOT BETWEEN :startDate AND :endDate order "
                        + "by e.joinDate");
        query.setParameter("startDate", localToTimeStamp(LocalDate.of(2005,1,1)));
        query.setParameter("endDate", localToTimeStamp(LocalDate.of(2018,1,1)));
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }


    private static Timestamp localToTimeStamp(LocalDate date){
        return Timestamp.from(date.atStartOfDay().toInstant(ZoneOffset.UTC));
    }
}